==> cds-services-impl-1.15.0-sources.jar: com/sap/cds/services/impl/messages/MessagesImpl.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.services.impl.messages;

import java.util.Locale;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.stream.Stream;

import com.sap.cds.services.ErrorStatus;
import com.sap.cds.services.ErrorStatuses;
import com.sap.cds.services.ServiceException;
import com.sap.cds.services.messages.Message;
import com.sap.cds.services.messages.Message.Severity;
import com.sap.cds.services.messages.Messages;

public class MessagesImpl implements Messages {

	private ConcurrentLinkedQueue<Message> messages = new ConcurrentLinkedQueue<>();

	private final Locale locale; // optional

	public MessagesImpl(Locale locale) {
		this.locale = locale;
	}

	@Override
	public Message info(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.INFO, locale, messageOrKey, args);
		messages.add(message);
		return message;
	}

	@Override
	public Message success(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.SUCCESS, locale, messageOrKey, args);
		messages.add(message);
		return message;
	}

	@Override
	public Message warn(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.WARNING, locale, messageOrKey, args);
		messages.add(message);
		return message;
	}

	@Override
	public Message error(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.ERROR, locale, messageOrKey, args);
		messages.add(message);
		return message;
	}

	@Override
	public void throwIfError() throws ServiceException {
		messages.stream().filter(m -> m.getSeverity().equals(Severity.ERROR)).findFirst().ifPresent(error -> {
			messages.remove(error);
			ErrorStatus status = new ErrorStatus() {
				@Override
				public String getCodeString() {
					return error.getCode() != null ? error.getCode() : String.valueOf(getHttpStatus());
				}
				@Override
				public int getHttpStatus() {
					return ErrorStatuses.BAD_REQUEST.getHttpStatus();
				}
			};

			ServiceException exception = new ServiceException(status, error.getMessage());
			exception.messageTarget(error.getTarget());
			throw exception;
		});
	}

	@Override
	public Stream<Message> stream() {
		return messages.stream();
	}

}

==> cds-services-impl-1.18.0-sources.jar: com/sap/cds/services/impl/messages/MessagesImpl.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.services.impl.messages;

import java.util.Locale;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.stream.Stream;

import com.sap.cds.services.ErrorStatus;
import com.sap.cds.services.ErrorStatuses;
import com.sap.cds.services.ServiceException;
import com.sap.cds.services.messages.Message;
import com.sap.cds.services.messages.Message.Severity;
import com.sap.cds.services.messages.Messages;
import com.sap.cds.services.runtime.CdsRuntime;

public class MessagesImpl implements Messages {

	private ConcurrentLinkedQueue<Message> messages = new ConcurrentLinkedQueue<>();

	private final CdsRuntime runtime;
	private final Locale locale; // optional

	public MessagesImpl(CdsRuntime runtime, Locale locale) {
		this.runtime = runtime;
		this.locale = locale;
	}

	@Override
	public Message info(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.INFO, runtime.getLocalizedMessage(messageOrKey, args, locale));
		messages.add(message);
		return message;
	}

	@Override
	public Message success(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.SUCCESS, runtime.getLocalizedMessage(messageOrKey, args, locale));
		messages.add(message);
		return message;
	}

	@Override
	public Message warn(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.WARNING, runtime.getLocalizedMessage(messageOrKey, args, locale));
		messages.add(message);
		return message;
	}

	@Override
	public Message error(String messageOrKey, Object... args) {
		Message message = new MessageImpl(Severity.ERROR, runtime.getLocalizedMessage(messageOrKey, args, locale));
		messages.add(message);
		return message;
	}

	@Override
	public void throwIfError() throws ServiceException {
		messages.stream().filter(m -> m.getSeverity().equals(Severity.ERROR)).findFirst().ifPresent(error -> {
			messages.remove(error);
			ErrorStatus status = new ErrorStatus() {
				@Override
				public String getCodeString() {
					return error.getCode() != null ? error.getCode() : String.valueOf(getHttpStatus());
				}
				@Override
				public int getHttpStatus() {
					return ErrorStatuses.BAD_REQUEST.getHttpStatus();
				}
			};

			ServiceException exception = new ServiceException(status, error.getMessage());
			exception.messageTarget(error.getTarget());
			throw exception;
		});
	}

	@Override
	public Stream<Message> stream() {
		return messages.stream();
	}

}
